/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_TAGS_HPP
#define GRAPH_MODEL_TAGS_HPP 1

#include <boost/graph/graph_traits.hpp>

namespace graph_model
{

  /// \brief Graph tags
  ///
  /// Graph tags are used to identify the type of a graph model.  Each
  /// one corresponds to a graph concept in the Boost Graph library.
  ///
  namespace tag
  {

    template < typename ... Tags >
    struct mixed_tag
      : Tags...
    {};

    // Boost.Graph graph concepts
    struct graph {};
    struct incidence_graph : boost::incidence_graph_tag , graph {};
    struct bidirectional_graph : boost::bidirectional_graph_tag , incidence_graph {};
    struct adjacency_graph : boost::adjacency_graph_tag , graph {};
    struct vertex_list_graph : boost::vertex_list_graph_tag , graph {};
    struct edge_list_graph : boost::edge_list_graph_tag , graph {};
    struct vertex_and_edge_list_graph : /* boost::vertex_and_edge_list_graph_tag, */ vertex_list_graph, edge_list_graph {};
    struct adjacency_matrix : boost::adjacency_matrix_tag , graph {};

    // Additional graph concepts
    struct incidence_and_vertex_list_graph : mixed_tag<incidence_graph,vertex_list_graph> {};

  } // namespace tag

} // namespace graph_model

#endif
