/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_EXPRESSION_HPP
#define GRAPH_MODEL_VERTEX_EXPRESSION_HPP 1

#include <graph_model/vertex/expression_fwd.hpp>
#include <type_traits>
#include <boost/yap/algorithm.hpp>
#include <graph_model/vertex/eval_impl_base.hpp>
#include <graph_model/vertex/expression_base.hpp>
#include <graph_model/vertex/expression_impl_base.hpp>
#include <graph_model/vertex/expression_terminal.hpp>
#include <graph_model/vertex/strategy/basic_expression.hpp>
#include <graph_model/vertex/strategy_owner_base.hpp>
#include <graph_model/vertex/tags.hpp>
#include <graph_model/vertex/vertex_expression.hpp>

namespace graph_model
{

  namespace vertex
  {

    /// Concrete expression
    ///
    template < typename Result >
    class expression
      : public eval_impl_base<expression<Result>
      , expression_impl_base<expression<Result>,tag::expression,expression_base<Result>
      > >
      , public strategy_owner_base<expression<Result>,strategy::basic_expression_factory<Result>>
    {};

    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<expression<T...>> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }
					   
    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<expression<T...>&> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }
					   
  } // namespace vertex

} // namespace graph_model

#endif
