/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_PDF_HPP
#define GRAPH_MODEL_VERTEX_PDF_HPP 1

#include <boost/yap/algorithm.hpp>
#include <graph_model/concept/function_concept.hpp>
#include <graph_model/concept/reference_wrapper_concept.hpp>
#include <graph_model/vertex/rvalue_reference_wrapper.hpp>
#include <graph_model/vertex/vertex_expression.hpp>
#include <mpl/enable_constructor.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Wrapper
	       , typename Variate = void
	       >
    struct pdf_wrapper
    {
      BOOST_CONCEPT_ASSERT((concept::reference_wrapper_concept<Wrapper>));
      BOOST_CONCEPT_ASSERT((concept::function_concept<Variate>));

      template < typename W, typename V >
      pdf_wrapper (W&& wrapper, V&& variate)
	: wrapper_(std::forward<W>(wrapper))
	, variate_(std::forward<V>(variate))
      {}

      template < typename ... T >
      auto operator () (T&& ... t) const { return wrapper_.get().pdf(variate_(std::forward<T>(t)...)); }

    private:
      Wrapper wrapper_;
      Variate variate_;
    };

    template < typename Wrapper >
    class pdf_wrapper< Wrapper,void >
    {
      BOOST_CONCEPT_ASSERT((concept::reference_wrapper_concept<Wrapper>));

      using self = pdf_wrapper;

    public:
      template < typename W
		 , typename = mpl::enable_constructor_t<self,W>
		 >
      explicit pdf_wrapper (W&& wrapper)
	: wrapper_(std::forward<W>(wrapper))
      {}

      template < typename ... T >
      auto operator () (T&& ... t) const
      {
	auto pdf = wrapper_.get().pdf();
	std::cout << "graph_model::vertex::pdf_wrapper<Wrapper,void>::operator()<T...>(T&&...)const: pdf=" << pdf << std::endl;
	return pdf;
	return wrapper_.get().pdf(std::forward<T>(t)...); }

    private:
      Wrapper wrapper_;
    };

    template < typename Expr, typename Variate >
    auto make_pdf_wrapper (std::reference_wrapper<Expr>&& wrapper, expression_function<Variate>&& variate)
    {
      using wrapper_type = std::reference_wrapper<Expr>;
      using variate_type = expression_function<Variate>;
      using function_type = pdf_wrapper<wrapper_type,variate_type>;
      return function_type(std::move(wrapper),std::move(variate));
    }

    template < typename Expr, typename Variate >
    auto make_pdf_wrapper (rvalue_reference_wrapper<Expr>&& wrapper, expression_function<Variate>&& variate)
    {
      using wrapper_type = rvalue_reference_wrapper<Expr>;
      using variate_type = expression_function<Variate>;
      using function_type = pdf_wrapper<wrapper_type,variate_type>;
      return function_type(std::move(wrapper),std::move(variate));
    }

    template < typename Expr, typename Variate >
    auto make_pdf_wrapper (Expr&& expr, Variate&& variate)
    {
      return make_pdf_wrapper
	(cref(std::forward<Expr>(expr)),
	 make_expression_function(std::forward<Variate>(variate)));
    }

    template < typename Expr >
    auto make_pdf_wrapper (std::reference_wrapper<Expr>&& wrapper)
    {
      using wrapper_type = std::reference_wrapper<Expr>;
      using function_type = pdf_wrapper<wrapper_type>;
      return function_type(wrapper);
    }

    template < typename Expr >
    auto make_pdf_wrapper (rvalue_reference_wrapper<Expr>&& wrapper)
    {
      using wrapper_type = rvalue_reference_wrapper<Expr>;
      using function_type = pdf_wrapper<wrapper_type>;
      return function_type(wrapper);
    }

    template < typename Expr >
    auto make_pdf_wrapper (Expr&& expr)
    {
      return make_pdf_wrapper(cref(std::forward<Expr>(expr)));
    }

    template < typename Expr, typename Variate >
    auto pdf (Expr&& expr, Variate&& variate)
    {
      using boost::yap::make_expression;
      using boost::yap::expr_kind;
      return make_expression<vertex_expression,expr_kind::call>
	(make_pdf_wrapper
	 (std::forward<Expr>(expr),
	  std::forward<Variate>(variate)));
    }

    template < typename Expr >
    auto pdf (Expr&& expr)
    {
      using boost::yap::make_expression;
      using boost::yap::expr_kind;
      return make_expression<vertex_expression,expr_kind::call>
	(make_pdf_wrapper(std::forward<Expr>(expr)));
    }

    template < typename Wrapper, typename Variate >
    auto pdf (pdf_wrapper<Wrapper,Variate> pdf) { return pdf; }

  } // namespace vertex

} // namespace graph_model

#endif
