/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file
/// \brief Simplify the initialization of Boost.Log sinks
///

#ifndef LOGGING_INITIALIZE_HPP
#define LOGGING_INITIALIZE_HPP 1

#include <boost/log/trivial.hpp>
#include <logging/initialize_trace_log.hpp>
#include <logging/logging_severity_level.hpp>

namespace logging
{

  template < typename Filter >
  auto initialize (std::ostream& os, Filter filter)
  {
    namespace keywords = boost::log::keywords;

    initialize_trace_log<>::apply
      (os,
       keywords::filter=filter,
       keywords::auto_flush=true
       );
  }

  auto initialize (std::ostream& os)
  { return initialize(os,boost::log::trivial::severity >= logging_severity_level::apply()); }

  template < typename Filter >
  auto initialize (std::string file_name, Filter filter)
  {
    namespace keywords = boost::log::keywords;

    initialize_trace_log<>::apply
      (keywords::file_name=file_name,
       keywords::filter=filter,
       keywords::auto_flush=true
       );
  }

  auto initialize (std::string file_name)
  { return initialize(file_name,boost::log::trivial::severity >= logging_severity_level::apply()); }

  auto initialize () { return initialize(std::cerr); }

} // namespace logging

#endif
