/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_distribution_normal_normal_distribution

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/distribution/normal/normal_distribution.hpp>
#include <graph_model/vertex/lit.hpp>

#include <graph_model/distribution/parameter/mean.hpp>
#include <graph_model/distribution/parameter/precision.hpp>
#include <graph_model/distribution/parameter/standard_deviation.hpp>
#include <graph_model/distribution/parameter/variance.hpp>
#include <graph_model/distribution/parameter/variate.hpp>

#include <iostream>
#include <string>

#if XXX_TOO_MANY_ARGUMENTS
BOOST_AUTO_TEST_CASE(test_too_many_arguments)
{
  using graph_model::distribution::normal::normal_distribution;
  auto distribution = normal_distribution(1,2,3,4);
}
#endif

BOOST_AUTO_TEST_CASE(test_default_constructor)
{
  using graph_model::distribution::normal::normal_distribution;
  using graph_model::distribution::parameter::mean;
  using graph_model::distribution::parameter::standard_deviation;
  using graph_model::distribution::parameter::variate;
  {
    auto distribution = normal_distribution();
    BOOST_CHECK_EQUAL(distribution.mean(),0);
    BOOST_CHECK_EQUAL(distribution.dispersion(),1);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),1);
    BOOST_CHECK_EQUAL(distribution.variate(),0);
  }
  {
    auto distribution = normal_distribution(42);
    BOOST_CHECK_EQUAL(distribution.mean(),42);
    BOOST_CHECK_EQUAL(distribution.dispersion(),1);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),1);
    BOOST_CHECK_EQUAL(distribution.variate(),0);
  }
  {
    auto distribution = normal_distribution(42,13);
    BOOST_CHECK_EQUAL(distribution.mean(),42);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),13);
    BOOST_CHECK_EQUAL(distribution.variate(),0);
  }
  {
    auto distribution = normal_distribution(42,13,17);
    BOOST_CHECK_EQUAL(distribution.mean(),42);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),13);
    BOOST_CHECK_EQUAL(distribution.variate(),17);
  }
  {
    auto distribution = normal_distribution(mean=42);
    BOOST_CHECK_EQUAL(distribution.mean(),42);
    BOOST_CHECK_EQUAL(distribution.dispersion(),1);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),1);
    BOOST_CHECK_EQUAL(distribution.variate(),0);
  }
  {
    auto distribution = normal_distribution(mean=42,13);
    BOOST_CHECK_EQUAL(distribution.mean(),42);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),13);
    BOOST_CHECK_EQUAL(distribution.variate(),0);
  }
  {
    auto distribution = normal_distribution(13,mean=42);
    BOOST_CHECK_EQUAL(distribution.mean(),42);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),13);
    BOOST_CHECK_EQUAL(distribution.variate(),0);
  }
  {
    auto distribution = normal_distribution(standard_deviation=2);
    BOOST_CHECK_EQUAL(distribution.mean(),0);
    BOOST_CHECK_EQUAL(distribution.dispersion(),2);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),2);
    BOOST_CHECK_EQUAL(distribution.variate(),0);
  }
  {
    auto distribution = normal_distribution(mean=1,standard_deviation=2);
    BOOST_CHECK_EQUAL(distribution.mean(),1);
    BOOST_CHECK_EQUAL(distribution.dispersion(),2);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),2);
    BOOST_CHECK_EQUAL(distribution.variate(),0);
  }
  {
    auto distribution = normal_distribution(mean=2,standard_deviation=3,variate=4);
    BOOST_CHECK_EQUAL(distribution.mean(),2);
    BOOST_CHECK_EQUAL(distribution.dispersion(),3);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),3);
    BOOST_CHECK_EQUAL(distribution.variate(),4);
  }
}

BOOST_AUTO_TEST_CASE(test_constructor)
{
  using graph_model::distribution::normal::normal_distribution;
  {
    auto distribution = normal_distribution();
    BOOST_CHECK_EQUAL(distribution.mean(),0);
    BOOST_CHECK_EQUAL(distribution.dispersion(),1);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),1);
    BOOST_CHECK_EQUAL(distribution.variate(),0);
  }
  {
    auto distribution = normal_distribution(0);
    BOOST_CHECK_EQUAL(distribution.mean(),0);
    BOOST_CHECK_EQUAL(distribution.dispersion(),1);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),1);
    BOOST_CHECK_EQUAL(distribution.variate(),0);
  }
  {
    auto distribution = normal_distribution(1,2);
    BOOST_CHECK_EQUAL(distribution.mean(),1);
    BOOST_CHECK_EQUAL(distribution.dispersion(),2);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),2);
    BOOST_CHECK_EQUAL(distribution.variate(),0);
  }
  {
    auto distribution = normal_distribution(2,3,4);
    BOOST_CHECK_EQUAL(distribution.mean(),2);
    BOOST_CHECK_EQUAL(distribution.dispersion(),3);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),3);
    BOOST_CHECK_EQUAL(distribution.variate(),4);
  }
}

BOOST_AUTO_TEST_CASE(test_constructor_lit)
{
  using graph_model::distribution::normal::normal_distribution;
  using graph_model::vertex::lit;

  auto mean = lit(42);
  auto distribution = normal_distribution(mean,1,2);
  BOOST_CHECK_EQUAL(distribution.mean(),42);
  BOOST_CHECK_EQUAL(distribution.dispersion(),1);
  BOOST_CHECK_EQUAL(distribution.standard_deviation(),1);
  BOOST_CHECK_EQUAL(distribution.variate(),2);
}

BOOST_AUTO_TEST_CASE(test_constructor_expr)
{
  using graph_model::distribution::normal::normal_distribution;
  using graph_model::vertex::lit;

  auto variate = 42;
  auto mean = lit(0) + 7;
  auto sd = 13 * lit(3);
  auto distribution = normal_distribution(mean,sd,variate);
  BOOST_CHECK_EQUAL(distribution.mean(),0 + 7);
  BOOST_CHECK_EQUAL(distribution.dispersion(),13 * 3);
  BOOST_CHECK_EQUAL(distribution.standard_deviation(),13 * 3);
  BOOST_CHECK_EQUAL(distribution.variate(),42);
}

BOOST_AUTO_TEST_CASE(test_constructor_tagged_parameters)
{
  using std::sqrt;

  using graph_model::distribution::normal::normal_distribution;
  using graph_model::distribution::parameter::mean;
  using graph_model::distribution::parameter::precision;
  using graph_model::distribution::parameter::standard_deviation;
  using graph_model::distribution::parameter::variance;
  using graph_model::distribution::parameter::variate;
  using graph_model::vertex::lit;

  auto v = 42;
  auto m = lit(0) + 7;
  auto dispersion = 13 * lit(3);

  {
    auto distribution
      = normal_distribution(variate=v,mean=m,standard_deviation=dispersion);
    BOOST_CHECK_EQUAL(distribution.mean(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = normal_distribution(variate=v,mean=m,variance=dispersion);
    BOOST_CHECK_EQUAL(distribution.mean(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),sqrt(13 * 3));
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = normal_distribution(variate=v,mean=m,precision=dispersion);
    BOOST_CHECK_EQUAL(distribution.mean(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),1.0 / (13 * 3));
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
}

BOOST_AUTO_TEST_CASE(test_constructor_tagged_parameter_pack)
{
  using graph_model::distribution::normal::normal_distribution;
  using graph_model::distribution::parameter::mean;
  using graph_model::distribution::parameter::precision;
  using graph_model::distribution::parameter::standard_deviation;
  using graph_model::distribution::parameter::variance;
  using graph_model::distribution::parameter::variate;
  using graph_model::vertex::lit;

  auto v = 42;
  auto m = lit(0) + 7;
  auto dispersion = 13 * lit(3);

  {
    auto distribution
      = normal_distribution(variate=v,mean=m,standard_deviation=dispersion);
    BOOST_CHECK_EQUAL(distribution.mean(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = normal_distribution(variate=v,mean=m,standard_deviation=dispersion);
    BOOST_CHECK_EQUAL(distribution.mean(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = normal_distribution(variate=v,mean=m,standard_deviation=dispersion);
    BOOST_CHECK_EQUAL(distribution.mean(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = normal_distribution(variate=v,m,dispersion);
    BOOST_CHECK_EQUAL(distribution.mean(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = normal_distribution(dispersion,mean=m,v);
    BOOST_CHECK_EQUAL(distribution.mean(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = normal_distribution(standard_deviation=dispersion,m,v);
    BOOST_CHECK_EQUAL(distribution.mean(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = normal_distribution(dispersion,mean=m,42);
    BOOST_CHECK_EQUAL(distribution.mean(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.dispersion(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.standard_deviation(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
}
