/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE graph_model_vertex_tagged_parameter

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/vertex/tagged_parameter.hpp>

#include <boost/mpl/vector.hpp>

struct tag {};

using test_types = boost::mpl::vector<int,long,float,double,long double>;

BOOST_AUTO_TEST_CASE_TEMPLATE(test_constructor_function,T,test_types)
{
  std::cout << "test_constructor_function" << std::endl;
  using graph_model::vertex::make_tagged_parameter;

  auto parameter = make_tagged_parameter<tag>(T(42));
  BOOST_CHECK_EQUAL(parameter(),42);
  std::cout << "test_constructor_function: done" << std::endl;
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_copy_constructor,T,test_types)
{
  std::cout << "test_copy_constructor" << std::endl;
  using graph_model::vertex::make_tagged_parameter;

  auto parameter = make_tagged_parameter<tag>(T(42));
  auto other = parameter;
  BOOST_CHECK_EQUAL(other(),42);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_move_constructor,T,test_types)
{
  std::cout << "test_move_constructor" << std::endl;
  using graph_model::vertex::make_tagged_parameter;

  auto parameter = make_tagged_parameter<tag>(T(42));
  auto other = std::move(parameter);
  BOOST_CHECK_EQUAL(other(),42);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_copy_assignment,T,test_types)
{
  std::cout << "test_copy_assignment" << std::endl;
  using graph_model::vertex::make_tagged_parameter;

  auto parameter = make_tagged_parameter<tag>(T(42));
  auto other = make_tagged_parameter<tag>(T(0));
  other = parameter;
  BOOST_CHECK_EQUAL(other(),42);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_move_assignment,T,test_types)
{
  std::cout << "test_move_assignment" << std::endl;
  using graph_model::vertex::make_tagged_parameter;

  auto parameter = make_tagged_parameter<tag>(T(42));
  auto other = make_tagged_parameter<tag>(T(0));
  other = std::move(parameter);
  BOOST_CHECK_EQUAL(other(),42);
}
